<?php

require_once 'base/GpealtesalarioEntityBase.php';

/**
 * Enter description here ...
 *
 * @package Entity
 *
 * @author  alessandrobartels
 */
class GpealtesalarioEntity extends GpeAltesalarioEntityBase
{
    private $_nrOrg;
    private $_dtAux;

    private $_alteracao;
    private $_persiste = false;

    const ALTERACAO_SALARIO_ADMISSAO = 1;


    /**
     * Enter description here ...
     *
     * @param integer $id    Identificador.
     * @param integer $nrOrg Numero da Organizacao de Trabalho.
     *
     * @author alessandrobartels
     */
    public function __construct($id = null, $nrOrg = null){
        parent::__construct($id);
        $this->_nrOrg = $nrOrg;
        $this->_dtAux = new TkDateTime();
    }


    /**
     * Enter description here ... (chamado no Reajuste).
     *
     * @param integer $nrvinculom Vinculo.
     * @param integer $nrOrg      Numero da Organizacao de Trabalho.
     *
     * @return GpealtesalarioEntity
     *
     * @author alessandrobartels
     */
    public function retornaAlteSalarioporVinculo($nrvinculom, $nrOrg){
         return Gpealtesalario::retornaAlteSalarioporVinculo($nrvinculom, $nrOrg);
    }


    /**
     * Retorna um array de instancias de GpealtesalarioEntity.
     *
     * @param TkDateTime $dtAlterSalario Data de altera��o de salario.
     * @param integer    $nrVinculom 	 Numero do vinculo.
     * @param integer 	 $nrModalidade   Numero da modalidade.
     *
     * @return $alteSalario
     *
     * @author alessandrobartels - 09/12/11
    */
    public function retornaAlteSalarioAtualPorVinculoModalidade(
        TkDateTime $dtAlterSalario, $nrVinculom, $nrModalidade
    ){

        $this->_dtAux->set($dtAlterSalario);
        $this->_dtAux->setDay(1)->addMonth(1)->subDay(1);

        return end(
            Gpealtesalario::retornaAlteSalarioAtualPorVinculoModalidade(
                $this->_nrOrg, $this->_dtAux, $nrVinculom, $nrModalidade
            )
        );
    }


    /**
     * Inicializa a lista com as alteracoes salarios (adiciona itens na lista).
     *
     * @param integer $nrOrg     Numero da Organizacao de Trabalho.
     * @param integer $nrVinculo Numero do vinculo.
     *
     * @return void
     *
     * @author alessandrobartels - 25/08/2011
     */
    private function _inicializaLista($nrOrg, $nrVinculo) {
        $alteracoes = Gpealtesalario::retornaAlteSalario($nrOrg, $nrVinculo);
        foreach ($alteracoes as $alteracao) {
            $this->addItem($alteracao);
        }
    }


    /**
     * Inicializa a lista com as alteracoes salarios em uma determinada data (adiciona itens na lista).
     *
     * @param integer    $nrOrg        Numero da Organizacao de Trabalho.
     * @param integer    $nrVinculo    Numero do vinculo.
     * @param TkDateTime $dtAltSalario Data de altaracao do salario.
     *
     * @return void
     *
     * @author alessandrobartels - 25/08/2011
     */
    private function _inicializaListaDataAlteracao($nrOrg, $nrVinculo, TkDateTime $dtAltSalario) {
        $alteracoes = Gpealtesalario::retornaAlteraSalarioPorData($nrOrg, $nrVinculo, $dtAltSalario);
        foreach ($alteracoes as $alteracao) {
            $this->addItem($alteracao);
        }
    }


    /**
     * Retorna lista de instancias de GpeAltesalarioEntity.
     *
     * @param integer $nrVinculo Numero do vinculo.
     *
     * @return array GpeAltesalarioEntity
     *
     * @author alessandrobartels - 16/12/2011
     */
    public function carregaListaAlteracaoSalario($nrVinculo){
        $itens = $this->getItems();
        if (count($itens) === 0) {
            $this->_inicializaLista($this->_nrOrg, $nrVinculo);
            $itens = $this->getItems();
        }

        return $itens;
    }


    /**
     * Retorna lista de instancias de GpeAltesalarioEntity em uma determinada data.
     *
     * @param TkDateTime $dtAltSalario Data de alteracao salarial.
     * @param integer    $nrVinculo    Numero do vinculo.
     *
     * @return array GpeAltesalarioEntity
     *
     * @author alessandrobartels - 16/12/2011
     */
    public function carregaListaAlteracaoSalarioData(TkDateTime $dtAltSalario, $nrVinculo){
        $itens = $this->getItems();
        if (count($itens) === 0) {
            $this->_inicializaListaDataAlteracao($this->_nrOrg, $nrVinculo, $dtAltSalario);
            $itens = $this->getItems();
        }

        return $itens;
    }


    /**
     * Carrega instancia de GpeAltesalarioEntity.
     *
     * @param TkDateTime $dtAltSalario Data de alteracao salarial.
     * @param integer    $nrVinculo    Numero do vinculo.
     * @param integer    $nrModalidade Numero da modalidade salarial.
     *
     * @return boolean
     *
     * @author alessandrobartels - 16/12/2011
     */
    public function carregaAlteracaoSalario(TkDateTime $dtAltSalario, $nrVinculo, $nrModalidade){
        $this->_alteracao = $this->retornaAlteSalarioAtualPorVinculoModalidade(
            $dtAltSalario, $nrVinculo, $nrModalidade
        );
        if (is_object($this->_alteracao) === true) {
            return true;
        } else {
            return false;
        }
    }


    /**
     * Retorna salario.
     *
     * @return float|NULL
     *
     * @author alessandrobartels - 13/12/2011
     */
    public function getSalario(){
        if (is_object($this->_alteracao) === true) {
            $vrSalario = $this->_alteracao->getVrsalario();
            return $vrSalario;
        }

        return null;
    }


    /**
     * Retorna tipo de salario.
     *
     * @return string|NULL
     *
     * @author alessandrobartels - 13/12/2011
     */
    public function getTipoSalario(){
        if (is_object($this->_alteracao) === true) {
            $tipoSalario = $this->_alteracao->getIdtpsalario();
            return $tipoSalario;
        }

        return null;
    }


    /**
     * Inclui Alteracao Salarial. (chamado VinculoLogic e Migracao).
     *
     * @param integer    $nrOrg            Numero da Organizacao.
     * @param integer    $nrVinculom       Numero do Vinculo.
     * @param float      $vrSalario        Valor do Salario.
     * @param float      $vrPercentual     Valor do percentual a ser computado.
     * @param float      $vrPercentualReal Valor do percentual real calculado.
     * @param float      $vrDiferencaReal  Valor da diferen�a da alteracao.
     * @param TkDateTime $dtAlteSalario    Data da alteracao salarial.
     * @param integer    $nrOcorrencia     Numero de Ocorrencia.
     * @param integer    $nrTipoAltera     Numero do tipo de alteracao salarial.
     * @param string     $idTipoSalario    Identificador do tipo de salario.
     * @param integer    $nrTpModalidade   Numero do tipo de modalidade.
     *
     * @throws Exception $e Erro na inclusao do registro.
     *
     * @return void
     *
     * @author alessandrobartels
     *         alessandrobartels - 15/12/2011 - inclusao do tipo de modalidade
     *         alessandrobartels - 19/12/2011 -
     */
    public function incluiAlteracaoSalario(
        $nrOrg, $nrVinculom, $vrSalario, $vrPercentual, $vrPercentualReal, $vrDiferencaReal,
        $dtAlteSalario, $nrOcorrencia, $nrTipoAltera, $idTipoSalario, $nrTpModalidade
    ) {
        //tk_warning('Vinculo: '.$nrVinculom.'   -   Data: '.$dtAlteSalario);

            $alteracao = new self();
            $alteracao->setNrorg($nrOrg);
            $alteracao->setNrvinculom($nrVinculom);
            $alteracao->setVrsalario($vrSalario);
            $alteracao->setVrpercreal($vrPercentual);
            $alteracao->setVrpercaltera($vrPercentualReal);
            $alteracao->setVrdiferencareal($vrDiferencaReal);
            $alteracao->setDtaltesalario($dtAlteSalario);
            $alteracao->setNrocorrencia($nrOcorrencia);
            $alteracao->setNrtipoaltera($nrTipoAltera);
            $alteracao->setIdtpsalario($idTipoSalario);
            $alteracao->setNrtpmodalidsal($nrTpModalidade);

            $alteracao->persist();

    }


    /**
     * Edita registro de alteracao de salario.
     *
     * @param float                $salarioReajustado  Valor do salario reajustado.
     * @param float                $vr_percentual      Valor do percentual a ser computado.
     * @param float                $vr_percentual_real Valor do percentual real calculado.
     * @param float                $vr_diferenca_real  Valor da diferenca da alteracao.
     * @param TkDateTime           $dt_alt_salario     Data de alteracao do salario.
     * @param integer              $nrModalidade       Numero do tipo de modalidade.
     * @param GpealtesalarioEntity $alteracao          Instancia de GpealtesalarioEntity.
     *
     * @return void
     *
     * @author alessandrobartels - 19/12/2011
     */
    public function editaAlteracaoSalario(
        $salarioReajustado, $vr_percentual, $vr_percentual_real, $vr_diferenca_real, TkDateTime $dt_alt_salario,
        $nrModalidade, GpealtesalarioEntity $alteracao
    ){
        try {

            $alteracao->setVrsalario($salarioReajustado);
            $alteracao->setVrpercaltera($vr_percentual);
            $alteracao->setVrpercreal($vr_percentual_real);
            $alteracao->setVrdiferencareal($vr_diferenca_real);
            $alteracao->setDtaltesalario($dt_alt_salario);
            $alteracao->setNrtpmodalidsal($nrModalidade);

            $alteracao->persist();

        }catch (Exception $e) {
            throw $e;
        }

    }


    /**
     * Realiza alteracao salarial. Cria nova alteracao quando nao existe no mes de ocorrencia.
     *
     * @param integer    $nrVinculo          Numero do vinculo.
     * @param float      $salarioReajustado  Valor do salario reajustado.
     * @param TkDateTime $dt_alt_salario     Data de alteracao salarial.
     * @param float      $vr_percentual      Valor do percentual a ser computado.
     * @param float      $vr_percentual_real Valor do percentual real calculado.
     * @param float      $vr_diferenca_real  Valor da diferen�a da alteracao.
     * @param integer    $nr_tipo_alteracao  Numero do tipo de alteracao salarial.
     * @param string     $nrTipoSalario      Identificador do tipo de salario.
     * @param integer    $nrModalidade       Numero do tipo de modalidade.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function geraAlteracaoSalarial(
        $nrVinculo, $salarioReajustado, TkDateTime $dt_alt_salario, $vr_percentual, $vr_percentual_real,
        $vr_diferenca_real, $nr_tipo_alteracao, $nrTipoSalario, $nrModalidade
    ) {
        $insere = true;
        $possuiMaiorValor = false;
        $posicao = -1;
        $nr_ocorrencia = 0;

        $alteracoes_salariais = $this->carregaListaAlteracaoSalario($nrVinculo);

        foreach ($alteracoes_salariais as $key => $alteracao) {
            if ((int)$alteracao->getNrtipoaltera() === (int)$nr_tipo_alteracao) {
                if ($dt_alt_salario->equals($alteracao->getDtaltesalario(), TkDateTime::FORMAT_DATE) === true) {
                    // Guarda a posicao na lista do registro que sera atualizado.
                    $posicao = $key;
                    $insere = false;
                    $nr_ocorrencia = $alteracao->getNrocorrencia();
                } elseif ($dt_alt_salario->isEarlier($alteracao->getDtaltesalario(), TkDateTime::FORMAT_DATE) === true
                ) {
                    /*
                     * Valida se existem registros futuros de alteracoes salariais com valores maiores aos do salario
                     * reajustado.
                     */
                    if ($salarioReajustado > $alteracao->getVrsalario()) {
                        $possuiMaiorValor = true;
                    }
                }
            }
        }

        if ($insere === true) {
            /* INSERCAO DE REGISTRO */
            $nr_ocorrencia++;

            // Exibe mensagem dizendo que encontrou registro futuro com valor menor.
            if ($possuiMaiorValor === true) {
                $sim = array(
                        'macro' => array(
                                    'class'  => new self,
                                    'method' => 'incluiAlteracaoSalario',
                                   ),
                        'args'  => array(
                                    $this->_nrOrg, $nrVinculo, $salarioReajustado, $vr_percentual, $vr_percentual_real,
                                    $vr_diferenca_real, $dt_alt_salario, $nr_ocorrencia, $nr_tipo_alteracao,
                                    $nrTipoSalario, $nrModalidade,
                                   ),
                        'LABEL' => 'Sim',
                );

                $nao = array(
                        'macro' => array(),
                        'args'  => array(),
                        'LABEL' => 'Nao',
                );

                $mensagem = 'Existe uma alteracao futura com valor menor ao valor a ser reajustado para o vinculo ';
                $mensagem .= $nrVinculo.'.<br/>  Deseja efetivar a alteracao assim mesmo?';
                tk_message_box(
                    $mensagem, array(
                                    $sim, $nao,
                                )
                );
            } else {
                $this->incluiAlteracaoSalario(
                    $this->_nrOrg, $nrVinculo, $salarioReajustado, $vr_percentual, $vr_percentual_real,
                    $vr_diferenca_real, $dt_alt_salario, $nr_ocorrencia, $nr_tipo_alteracao, $nrTipoSalario,
                    $nrModalidade
                );
            }
        } else {
            /* ALTERACAO DE REGISTRO */
            $alteracao = $this->getItem($posicao);

            // Exibe mensagem dizendo que encontrou registro futuro com valor menor.
            if ($possuiMaiorValor === true) {
                $sim = array(
                        'macro' => array(
                                    'class'  => new self,
                                    'method' => 'editaAlteracaoSalario',
                                   ),
                        'args'  => array(
                                    $salarioReajustado, $vr_percentual, $vr_percentual_real, $vr_diferenca_real,
                                    $dt_alt_salario, $nrModalidade, $alteracao,
                                   ),
                        'LABEL' => 'Sim',
                );

                $nao = array(
                        'macro' => array(),
                        'args'  => array(),
                        'LABEL' => 'Nao',
                );

                $mensagem = 'Existe uma alteracao futura com valor menor ao valor a ser reajustado para o vinculo ';
                $mensagem .= $nrVinculo.'.<br/>  Deseja efetivar a alteracao assim mesmo?';
                tk_message_box(
                    $mensagem, array(
                                    $sim, $nao,
                                )
                );
            } else {
                $this->editaAlteracaoSalario(
                    $salarioReajustado, $vr_percentual, $vr_percentual_real, $vr_diferenca_real, $dt_alt_salario,
                    $nrModalidade, $alteracao
                );
            }
        }
    }


    /**
     * Override TkEntity::persist().
     *
     * @see    TkEntity::persist()
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function persist() {
        try {
            if ($this->_persiste === true) {
                $this->_persiste = false;
                $this->_alteracao->persist();
            } else {
                parent::persist();
            }
        } catch (Exception $e) {
            throw new Exception(
                'Erro na inclusao do novo registro - GPEALTESALARIO'.$e->getMessage(), $code, $previous
            );
        }

    }

    public function setAlteracaoSalario($nrOrg, $nrVinculom, $vrSalario, $vrPercentual, $vrPercentualReal,
                                           $vrDiferencaReal, $dtAlteSalario, $nrOcorrencia, $nrTipoAltera, $idTipoSalario,
                                           $nrTpModalidade)
    {

        $this->setNrorg($nrOrg);
        $this->setNrvinculom($nrVinculom);
        $this->setVrsalario($vrSalario);
        $this->setVrpercreal($vrPercentual);
        $this->setVrpercaltera($vrPercentualReal);
        $this->setVrdiferencareal($vrDiferencaReal);
        $this->setDtaltesalario($dtAlteSalario);
        $this->setNrocorrencia($nrOcorrencia);
        $this->setNrtipoaltera($nrTipoAltera);
        $this->setIdtpsalario($idTipoSalario);
        $this->setNrtpmodalidsal($nrTpModalidade);

    }

    public static function getByNrorgVinculomDataOcorrencia($nrorg, $nrvinculom, $dtaltesalario, $nrocorrencia) {
        $altesalario = self::getEntityByWhere(
            'NRORG = ? AND NRVINCULOM = ? AND DTALTESALARIO = ? AND NROCORRENCIA = ?',
            array($nrorg, $nrvinculom, $dtaltesalario, $nrocorrencia)
        );
        return $altesalario;
    }

}