<?php
/**
 * GepSituacaoLogic.
 *
 * @package Logic
 * @author  daniellepereira <username@example.com>
 */
class GepSituacaoLogic {


    private $_GepSituacaoEntity;


    /**
     * Construtor da classe.
     */
    public function __construct(){
        $this->_GepSituacaoEntity = new GepSituacaoEntity();
    }


    /**
    * Descri?: Gera o novo c?o de Situacao por Tipo de Fluxo.
    *
    * @param integer $nrorg N?o da Organiza?.
    *
    * @return $codigo integer
    */
    public static function retornaProxNumeroSituacao($nrorg){
        $codigo = GepSituacaoEntity::retornaProxNumeroSituacao($nrorg);
        return $codigo;
    }


    /**
     * Retorna dados de uma determinada situa?.
     *
     * @param integer $nrorg       N?o da Organiza?.
     * @param integer $nrsituacao  N?o da Situa?.
     * @param integer $nrtipofluxo Tipo de fluxo.
     *
     * @return Ambigous <Ambigous, NULL, mixed>
     */
    public static function retornaSituacao($nrorg, $nrsituacao, $nrtipofluxo){
        try {
            $dados = GepSituacaoEntity::retornaSituacao(
                $nrorg,
                $nrsituacao,
                $nrtipofluxo
            );

            return $dados;
        } catch (Exception $e) {

            switch ($e->getCode())
            {
            case GepSituacaoEntity::RESULTADO_SEM_REGISTRO:
                return GepSituacaoEntity::RESULTADO_SEM_REGISTRO;
            default:
                tk_message('Erro desconhecido: '.$e->getMessage());
                return false;
            }

            return false;
        }
    }


    /**
     * Retorna os poss?is parceiros respons?is pela situa?.
     *
     * @param integer $nrorg       N?o da Organiza?.
     * @param integer $situacao    N?o da situa?.
     * @param integer $parceiro    N?o do Parceiro.
     * @param integer $solicitacao N?o da Solicita?.
     *
     * @return array
     */
    public static function retornaParcRespTramitacao($nrorg, $situacao, $parceiro, $solicitacao){
        try {
            $dados = GepSituacaoEntity::retornaParcRespTramitacao($nrorg, $situacao, $parceiro, $solicitacao);
            return $dados;
        } catch (Exception $e) {
            switch ($e->getCode()){
            case GepSituacaoEntity::SEM_PARCEIROS:
                return GepSituacaoEntity::SEM_PARCEIROS;
            default:
                tk_warning($e->getMessage());
                return false;
            }
        }
    }


    /**
     * Retorna as situacoes de determinado tipo de fluxo.
     *
     * @param integer $nrorg       Numero da organizacao.
     * @param integer $nrtipofluxo Numero do tipo de fluxo.
     *
     * @return $consulta            Resultado da query.
     */
    public static function retornaSituacaoPorTipoFluxo($nrorg, $nrtipofluxo){
        try{
            $consulta = GepSituacaoEntity::retornaSituacaoPorTipoFluxo($nrorg, $nrtipofluxo);
            return $consulta;

        } catch(Exception $e) {
            switch ($e->getCode())
            {
            case GepSituacaoEntity::RESULTADO_SEM_REGISTRO;
                return false;
            default:
                tk_message('Erro desconhecido: '.$e->getMessage());
            }
        }
    }


    /**       ok
     * Retorna o SQL para o search dos operadores responsaveis em uma situacao.
     *
     * @param integer $nrorg       Numero da organizacao.
     * @param integer $situacao    Numero da situacao atual.
     * @param integer $parceiro    Codigo do parceiro.
     * @param integer $solicitacao Numero da Solicitacao.
     *
     * @return string
     */
    public function retornaSqlOperRespSituacao($nrorg, $situacao, $parceiro, $solicitacao){
        $sql = $this->_GepSituacaoEntity->retornaSqlOperRespSituacao($nrorg, $situacao, $parceiro, $solicitacao);
        return $sql;
    }


    /**        ok
     * Retorna o sql com os perfis de acesso por uma determinada situacao.
     *
     * @param integer $nrorg    Numero da Organizacao.
     * @param integer $situacao Codigo da situacao.
     *
     * @return string
     */
    public function retornaSqlPerfilAcessoSituacao($nrorg, $situacao){
        $perfil = $this->_GepSituacaoEntity->retornaSqlPerfilAcessoSituacao($nrorg, $situacao);
        return $perfil;
    }


    /**
     * Retorna a pergunta parametrizada ou nao para a tramitacao de uma solicitacao.
     *
     * @param integer $nrorg       Numero da organizacao.
     * @param integer $nrsituacao  Numero da situacao.
     * @param integer $nrtipofluxo Tipo de fluxo.
     *
     * @return string
     */
    public function retornaPerguntaTramitacao($nrorg, $nrsituacao, $nrtipofluxo){
        $dados_situacao = GepSituacaoLogic::retornaSituacao($nrorg, $nrsituacao, $nrtipofluxo);
        if ($dados_situacao !== GepSituacaoEntity::RESULTADO_SEM_REGISTRO AND $dados_situacao !== false) {
            $pergunta = $dados_situacao->getTxperguntaenca();
        }

        return $pergunta;
    }
}